@@ -5,6 +5,7 @@
 import atexit
 import logging
 import ntpath
+import typing
 import uuid
 import warnings
 
@@ -54,14 +55,25 @@ class _ConfigSingleton(type):
 
     def __call__(cls, *args, **kwargs):
         if cls not in cls.__instances:
-            cls.__instances[cls] = super(_ConfigSingleton, cls).__call__(*args, **kwargs)
+            config = super(_ConfigSingleton, cls).__call__(*args, **kwargs)
+            cls.__instances[cls] = config
+
+            # Needs to be done after the config instance is in the singleton dict due to setting this value will kick
+            # off an IPC connection which then gets and instance of this config to see what the default creds are.
+            # The property will not run the DFS request if the passed in value matches what is already there. Set
+            # to None initially to ensure it detects a changed value.
+            # https://github.com/jborean93/smbprotocol/issues/109
+            dc_value = config.domain_controller
+            config.domain_controller = None
+            config.domain_controller = dc_value
 
         else:
             # Allow users to initialise and set multiple config options like ConfigType(key=value) even when the object
             # has been initialized already.
-            cls.__instances[cls].set(**kwargs)
+            config = cls.__instances[cls]
+            config.set(**kwargs)
 
-        return cls.__instances[cls]
+        return config
 
 
 class ClientConfig(object, metaclass=_ConfigSingleton):
@@ -95,13 +107,10 @@ class ClientConfig(object, metaclass=_ConfigSingleton):
         self.skip_dfs = skip_dfs
         self.auth_protocol = auth_protocol
         self.require_secure_negotiate = require_secure_negotiate
-        self._domain_controller = None  # type: Optional[str]
+        self._domain_controller = domain_controller  # type: Optional[str]
         self._domain_cache = []  # type: List[DomainEntry]
         self._referral_cache = []  # type: List[ReferralEntry]
 
-        # This relies on other attributes so set this last
-        self.domain_controller = domain_controller
-
     @property
     def domain_controller(self):
         return self._domain_controller
@@ -136,14 +145,16 @@ class ClientConfig(object, metaclass=_ConfigSingleton):
         self._referral_cache.append(ReferralEntry(referral))
 
     def lookup_domain(self, domain_name):  # type: (str) -> Optional[DomainEntry]
+        # TODO: Check domain referral expiry and resend request if expired
         for domain in self._domain_cache:
             if domain.domain_name.lower() == ("\\" + domain_name.lower()):
                 return domain
 
-    def lookup_referral(self, path_components):  # type: (List[str]) -> Optional[ReferralEntry]
+    def lookup_referral(self, path_components: typing.List[str]) -> typing.Optional[ReferralEntry]:
         """ Checks if the path exists in the DFS referral cache. """
         # A lookup in ReferralCache involves searching for an entry with DFSPathPrefix that is a complete prefix of the
         # path being looked up.
+        self._clear_expired_cache()
         hits = []
         for referral in self._referral_cache:
             referral_path_components = [p for p in referral.dfs_path.split("\\") if p]
@@ -176,6 +187,9 @@ class ClientConfig(object, metaclass=_ConfigSingleton):
         if domain_controller:
             self.domain_controller = config['domain_controller']
 
+    def _clear_expired_cache(self) -> None:
+        self._referral_cache = [refferal for refferal in self._referral_cache if not refferal.is_expired]
+
 
 def dfs_request(tree, path):  # type: (TreeConnect, str) -> DFSReferralResponse
     """ Send a DFS Referral request to the IPC tree and return the referrals. """
